@@ -17,7 +17,7 @@ use crate::{
         mapx_ord_rawvalue::MapxOrdRawValue, mapx_raw::MapxRaw, orphan::Orphan,
         vecx::Vecx, vecx_raw::VecxRaw,
     },
-    BranchName, ParentBranchName, VersionName,
+    BranchName, BranchNameOwned, ParentBranchName, VersionName, VersionNameOwned,
 };
 use ruc::*;
 use std::{
@@ -94,6 +94,26 @@ pub trait VsMgmt {
         branch_name: BranchName,
     ) -> Result<()>;
 
+    fn version_exists_globally(&self, version_name: VersionName) -> bool;
+
+    fn version_list(&self) -> Result<Vec<VersionNameOwned>>;
+
+    fn version_list_by_branch(
+        &self,
+        branch_name: BranchName,
+    ) -> Result<Vec<VersionNameOwned>>;
+
+    fn version_list_globally(&self) -> Vec<VersionNameOwned>;
+
+    fn version_has_change_set(&self, version_name: VersionName) -> Result<bool>;
+
+    fn version_clean_up_globally(&self) -> Result<()>;
+
+    /// # Safety
+    ///
+    /// Version itself and its corresponding changes will be completely purged from all branches
+    unsafe fn version_revert_globally(&self, version_name: VersionName) -> Result<()>;
+
     /// Create a new branch based on the head of the default branch.
     fn branch_create(
         &self,
@@ -218,6 +238,26 @@ pub trait VsMgmt {
     /// all default operations will be applied to it.
     fn branch_set_default(&mut self, branch_name: BranchName) -> Result<()>;
 
+    fn branch_is_empty(&self, branch_name: BranchName) -> Result<bool>;
+
+    fn branch_list(&self) -> Vec<BranchNameOwned>;
+
+    fn branch_get_default(&self) -> BranchNameOwned;
+
+    /// Logically similar to `std::ptr::swap`
+    ///
+    /// For example: If you have a master branch and a test branch, the data is always trial-run on the test branch, and then periodically merged back into the master branch. Rather than merging the test branch into the master branch, and then recreating the new test branch, it is more efficient to just swap the two branches, and then recreating the new test branch.
+    ///
+    /// # Safety
+    ///
+    /// - Non-'thread safe'
+    /// - Must ensure that there are no reads and writes to these two branches during the execution
+    unsafe fn branch_swap(
+        &mut self,
+        branch_1: BranchName,
+        branch_2: BranchName,
+    ) -> Result<()>;
+
     /// Clean outdated versions out of the default branch.
     fn prune(&self, reserved_ver_num: Option<usize>) -> Result<()>;
 }
@@ -315,6 +355,47 @@ macro_rules! impl_vs_methods {
                 .c(d!())
         }
 
+        #[inline(always)]
+        fn version_exists_globally(&self, version_name: VersionName) -> bool {
+            self.inner.version_exists_globally(version_name)
+        }
+
+        #[inline(always)]
+        fn version_list(&self) -> Result<Vec<VersionNameOwned>> {
+            self.inner.version_list().c(d!())
+        }
+
+        #[inline(always)]
+        fn version_list_by_branch(
+            &self,
+            branch_name: BranchName,
+        ) -> Result<Vec<VersionNameOwned>> {
+            self.inner.version_list_by_branch(branch_name).c(d!())
+        }
+
+        #[inline(always)]
+        fn version_list_globally(&self) -> Vec<VersionNameOwned> {
+            self.inner.version_list_globally()
+        }
+
+        #[inline(always)]
+        fn version_has_change_set(&self, version_name: VersionName) -> Result<bool> {
+            self.inner.version_has_change_set(version_name).c(d!())
+        }
+
+        #[inline(always)]
+        fn version_clean_up_globally(&self) -> Result<()> {
+            self.inner.version_clean_up_globally().c(d!())
+        }
+
+        #[inline(always)]
+        unsafe fn version_revert_globally(
+            &self,
+            version_name: VersionName,
+        ) -> Result<()> {
+            self.inner.version_revert_globally(version_name).c(d!())
+        }
+
         /// Create a new branch based on the head of the default branch.
         #[inline(always)]
         fn branch_create(
@@ -514,6 +595,26 @@ macro_rules! impl_vs_methods {
             self.inner.branch_set_default(branch_name).c(d!())
         }
 
+        fn branch_is_empty(&self, branch_name: BranchName) -> Result<bool> {
+            self.inner.branch_is_empty(branch_name).c(d!())
+        }
+
+        fn branch_list(&self) -> Vec<BranchNameOwned> {
+            self.inner.branch_list()
+        }
+
+        fn branch_get_default(&self) -> BranchNameOwned {
+            self.inner.branch_get_default()
+        }
+
+        unsafe fn branch_swap(
+            &mut self,
+            branch_1: BranchName,
+            branch_2: BranchName,
+        ) -> Result<()> {
+            self.inner.branch_swap(branch_1, branch_2).c(d!())
+        }
+
         /// Clean outdated versions out of the default reserved number.
         #[inline(always)]
         fn prune(&self, reserved_ver_num: Option<usize>) -> Result<()> {
@@ -575,6 +676,37 @@ macro_rules! impl_vs_methods_nope {
             Ok(())
         }
 
+        fn version_exists_globally(&self, _: VersionName) -> bool {
+            true
+        }
+
+        fn version_list(&self) -> Result<Vec<VersionNameOwned>> {
+            Ok(Default::default())
+        }
+
+        fn version_list_by_branch(
+            &self,
+            _: BranchName,
+        ) -> Result<Vec<VersionNameOwned>> {
+            Ok(Default::default())
+        }
+
+        fn version_list_globally(&self) -> Vec<VersionNameOwned> {
+            Default::default()
+        }
+
+        fn version_has_change_set(&self, _: VersionName) -> Result<bool> {
+            Ok(true)
+        }
+
+        fn version_clean_up_globally(&self) -> Result<()> {
+            Ok(())
+        }
+
+        unsafe fn version_revert_globally(&self, _: VersionName) -> Result<()> {
+            Ok(())
+        }
+
         #[inline(always)]
         fn branch_create(&self, _: BranchName, _: VersionName) -> Result<()> {
             Ok(())
@@ -669,6 +801,22 @@ macro_rules! impl_vs_methods_nope {
             Ok(())
         }
 
+        fn branch_is_empty(&self, _: BranchName) -> Result<bool> {
+            Ok(true)
+        }
+
+        fn branch_list(&self) -> Vec<BranchNameOwned> {
+            Default::default()
+        }
+
+        fn branch_get_default(&self) -> BranchNameOwned {
+            Default::default()
+        }
+
+        unsafe fn branch_swap(&mut self, _: BranchName, _: BranchName) -> Result<()> {
+            Ok(())
+        }
+
         #[inline(always)]
         fn prune(&self, _: Option<usize>) -> Result<()> {
             Ok(())
@@ -887,6 +1035,58 @@ impl<T: VsMgmt> VsMgmt for Option<T> {
         Ok(())
     }
 
+    fn version_exists_globally(&self, version_name: VersionName) -> bool {
+        if let Some(i) = self.as_ref() {
+            return i.version_exists_globally(version_name);
+        }
+        true
+    }
+
+    fn version_list(&self) -> Result<Vec<VersionNameOwned>> {
+        if let Some(i) = self.as_ref() {
+            i.version_list().c(d!())?;
+        }
+        Ok(Default::default())
+    }
+
+    fn version_list_by_branch(
+        &self,
+        branch_name: BranchName,
+    ) -> Result<Vec<VersionNameOwned>> {
+        if let Some(i) = self.as_ref() {
+            i.version_list_by_branch(branch_name).c(d!())?;
+        }
+        Ok(Default::default())
+    }
+
+    fn version_list_globally(&self) -> Vec<VersionNameOwned> {
+        if let Some(i) = self.as_ref() {
+            return i.version_list_globally();
+        }
+        Default::default()
+    }
+
+    fn version_has_change_set(&self, version_name: VersionName) -> Result<bool> {
+        if let Some(i) = self.as_ref() {
+            i.version_has_change_set(version_name).c(d!())?;
+        }
+        Ok(true)
+    }
+
+    fn version_clean_up_globally(&self) -> Result<()> {
+        if let Some(i) = self.as_ref() {
+            i.version_clean_up_globally().c(d!())?;
+        }
+        Ok(())
+    }
+
+    unsafe fn version_revert_globally(&self, version_name: VersionName) -> Result<()> {
+        if let Some(i) = self.as_ref() {
+            i.version_revert_globally(version_name).c(d!())?;
+        }
+        Ok(())
+    }
+
     #[inline(always)]
     fn branch_create(
         &self,
@@ -1061,6 +1261,38 @@ impl<T: VsMgmt> VsMgmt for Option<T> {
         Ok(())
     }
 
+    fn branch_is_empty(&self, branch_name: BranchName) -> Result<bool> {
+        if let Some(i) = self.as_ref() {
+            i.branch_is_empty(branch_name).c(d!())?;
+        }
+        Ok(true)
+    }
+
+    fn branch_list(&self) -> Vec<BranchNameOwned> {
+        if let Some(i) = self.as_ref() {
+            return i.branch_list();
+        }
+        Default::default()
+    }
+
+    fn branch_get_default(&self) -> BranchNameOwned {
+        if let Some(i) = self.as_ref() {
+            return i.branch_get_default();
+        }
+        Default::default()
+    }
+
+    unsafe fn branch_swap(
+        &mut self,
+        branch_1: BranchName,
+        branch_2: BranchName,
+    ) -> Result<()> {
+        if let Some(i) = self.as_mut() {
+            i.branch_swap(branch_1, branch_2).c(d!())?;
+        }
+        Ok(())
+    }
+
     #[inline(always)]
     fn prune(&self, reserved_ver_num: Option<usize>) -> Result<()> {
         if let Some(i) = self.as_ref() {
